@@ -1,19 +1,19 @@
 import {DOMSerializer} from "prosemirror-model"
 import download from "downloadjs"
+import pretty from "pretty"
 
 import {BIBLIOGRAPHY_HEADERS} from "../../../schema/i18n"
 
 import {getMissingChapterData, uniqueObjects} from "../tools"
 import {epubBookOpfTemplate, epubBookCoverTemplate, epubBookTitlepageTemplate,
-  epubBookCopyrightTemplate} from "./templates"
+    epubBookCopyrightTemplate} from "./templates"
 import {mathliveOpfIncludes} from "../../../mathlive/opf_includes"
 import {DOMExporter} from "../../../exporter/tools/dom_export"
-import {setLinks, orderLinks, getTimestamp, styleEpubFootnotes, addFigureLabels} from "../../../exporter/epub/tools"
+import {setLinks, orderLinks, getTimestamp, styleEpubFootnotes, addCategoryLabels} from "../../../exporter/epub/tools"
 
-import {ncxTemplate, ncxItemTemplate, navTemplate, navItemTemplate,
-  containerTemplate, xhtmlTemplate} from "../../../exporter/epub/templates"
+import {ncxTemplate, ncxItemTemplate, navTemplate, containerTemplate, xhtmlTemplate} from "../../../exporter/epub/templates"
 import {node2Obj, obj2Node} from "../../../exporter/tools/json"
-import {removeHidden} from "../../../exporter/tools/doc_contents"
+import {removeHidden} from "../../../exporter/tools/doc_content"
 import {modifyImages} from "../../../exporter/tools/html"
 import {createSlug} from "../../../exporter/tools/file"
 import {ZipFileCreator} from "../../../exporter/tools/zip"
@@ -49,19 +49,26 @@ export class EpubBookExporter extends DOMExporter {
     }
 
     addBookStyle() {
-        const bookStyle = this.documentStyles.find(bookStyle => bookStyle.slug===this.book.settings.book_style)
+        const bookStyle = this.documentStyles.find(bookStyle => bookStyle.slug === this.book.settings.book_style)
         if (!bookStyle) {
             return false
         }
         // The files will be in the base directory. The filenames of
         // BookStyleFiles will therefore not need to replaced with their URLs.
+        let contents = bookStyle.contents
+        bookStyle.bookstylefile_set.forEach(
+            ([_url, filename]) => contents = contents.replace(
+                new RegExp(filename, 'g'),
+                `media/${filename}`
+            )
+        )
 
-        this.styleSheets.push({contents: bookStyle.contents, filename: `${bookStyle.slug}.css`})
+        this.styleSheets.push({contents, filename: `css/${bookStyle.slug}.css`})
         this.fontFiles = this.fontFiles.concat(bookStyle.bookstylefile_set.map(([url, filename]) => ({
-            filename,
+            filename: `css/media/${filename}`,
             url
         })))
-        return `${bookStyle.slug}.css`
+        return `css/${bookStyle.slug}.css`
     }
 
     exportOne() {
@@ -95,13 +102,14 @@ export class EpubBookExporter extends DOMExporter {
             level: 0,
             subItems: [],
         })
+        let currentPart = false
         this.chapters = this.book.chapters.map(chapter => {
             const doc = this.docList.find(doc => doc.id === chapter.text),
                 schema = this.schema
             schema.cached.imageDB = {db: doc.images}
-            const docContents = removeHidden(doc.contents),
+            const docContent = removeHidden(doc.content),
                 serializer = DOMSerializer.fromSchema(schema),
-                tempNode = serializer.serializeNode(schema.nodeFromJSON(docContents))
+                tempNode = serializer.serializeNode(schema.nodeFromJSON(docContent))
             const contentsEl = document.createElement('body')
             let math = false
             while (tempNode.firstChild) {
@@ -109,7 +117,7 @@ export class EpubBookExporter extends DOMExporter {
             }
 
             this.images = this.images.concat(modifyImages(contentsEl))
-            addFigureLabels(contentsEl, doc.settings.language)
+            addCategoryLabels(contentsEl, doc.settings.language)
             const equations = contentsEl.querySelectorAll('.equation')
 
             const figureEquations = contentsEl.querySelectorAll('.figure-equation')
@@ -119,7 +127,7 @@ export class EpubBookExporter extends DOMExporter {
                 this.math = true
             }
 
-            if (chapter.part && chapter.part.length) {
+            if (chapter.part?.length) {
                 this.contentItems.push({
                     link: `document-${chapter.number}.xhtml`,
                     title: chapter.part,
@@ -128,6 +136,7 @@ export class EpubBookExporter extends DOMExporter {
                     level: -1,
                     subItems: [],
                 })
+                currentPart = chapter.part
             }
 
             // Make links to all H1-3 and create a TOC list of them
@@ -137,7 +146,8 @@ export class EpubBookExporter extends DOMExporter {
 
             return {
                 contents: contentsEl,
-                number : chapter.number,
+                number: chapter.number,
+                currentPart,
                 part: chapter.part,
                 math,
                 doc
@@ -160,10 +170,10 @@ export class EpubBookExporter extends DOMExporter {
                     if (bibHTML.length > 0) {
                         chapter.contents.innerHTML += bibHTML
                     }
-                    this.contents = chapter.contents
+                    this.content = chapter.contents
                     this.cleanHTML(citRenderer.fm)
-                    chapter.contents = this.contents
-                    delete this.contents
+                    chapter.contents = this.content
+                    delete this.content
                     return Promise.resolve()
                 }
             )
@@ -176,12 +186,13 @@ export class EpubBookExporter extends DOMExporter {
         this.outputList = this.outputList.concat(
             this.chapters.map(chapter => {
                 chapter.contents = styleEpubFootnotes(chapter.contents)
-                const styleSheets = [{filename: 'document.css'}]
+                const styleSheets = [{filename: 'css/document.css'}]
                 if (bookStyle) {
                     styleSheets.push({filename: bookStyle})
                 }
                 let xhtmlCode = xhtmlTemplate({
                     part: chapter.part,
+                    currentPart: chapter.currentPart,
                     shortLang: chapter.doc.settings.language.split('-')[0],
                     title: chapter.doc.title,
                     metadata: chapter.doc.metadata,
@@ -194,7 +205,7 @@ export class EpubBookExporter extends DOMExporter {
 
                 return {
                     filename: `EPUB/document-${chapter.number}.xhtml`,
-                    contents: xhtmlCode
+                    contents: pretty(xhtmlCode, {ocd: true})
                 }
             })
         )
@@ -262,33 +273,33 @@ export class EpubBookExporter extends DOMExporter {
         const navCode = navTemplate({
             shortLang,
             contentItems: this.contentItems,
-            templates: {navTemplate, navItemTemplate}
+            styleSheets: this.styleSheets
         })
 
         this.outputList = this.outputList.concat([{
             filename: 'META-INF/container.xml',
-            contents: containerTemplate({})
+            contents: pretty(containerTemplate({}), {ocd: true})
         }, {
             filename: 'EPUB/document.opf',
-            contents: opfCode
+            contents: pretty(opfCode, {ocd: true})
         }, {
             filename: 'EPUB/document.ncx',
-            contents: ncxCode
+            contents: pretty(ncxCode, {ocd: true})
         }, {
             filename: 'EPUB/document-nav.xhtml',
-            contents: navCode
+            contents: pretty(navCode, {ocd: true})
         }, {
             filename: 'EPUB/titlepage.xhtml',
-            contents: epubBookTitlepageTemplate({
+            contents: pretty(epubBookTitlepageTemplate({
                 book: this.book
-            })
+            }), {ocd: true})
         }, {
             filename: 'EPUB/copyright.xhtml',
-            contents: epubBookCopyrightTemplate({
+            contents: pretty(epubBookCopyrightTemplate({
                 book: this.book,
                 creator: this.user.name,
                 languages
-            })
+            }), {ocd: true})
         }])
 
         this.outputList = this.outputList.concat(
@@ -307,7 +318,7 @@ export class EpubBookExporter extends DOMExporter {
 
         this.fontFiles.forEach(font => {
             this.binaryFiles.push({
-                filename: 'EPUB/' + font.filename,
+                filename: `EPUB/${font.filename}`,
                 url: font.url
             })
         })
@@ -316,7 +327,7 @@ export class EpubBookExporter extends DOMExporter {
 
         if (this.math) {
             this.includeZips.push({
-                'directory': 'EPUB',
+                'directory': 'EPUB/css',
                 'url': `${settings_STATIC_URL}zip/mathlive_style.zip?v=${transpile_VERSION}`
             })
         }
