@@ -1,6 +1,6 @@
 import {getMissingChapterData, uniqueObjects} from "../tools"
 import {htmlBookExportTemplate, htmlBookIndexTemplate} from "./templates"
-import {removeHidden} from "../../../exporter/tools/doc_contents"
+import {removeHidden} from "../../../exporter/tools/doc_content"
 import {setLinks, orderLinks} from "../../../exporter/epub/tools"
 import {DOMExporter} from "../../../exporter/tools/dom_export"
 import {createSlug} from "../../../exporter/tools/file"
@@ -8,9 +8,10 @@ import {modifyImages} from "../../../exporter/tools/html"
 import {ZipFileCreator} from "../../../exporter/tools/zip"
 import {RenderCitations} from "../../../citations/render"
 import {addAlert} from "../../../common"
-import {BIBLIOGRAPHY_HEADERS, FIG_CATS} from "../../../schema/i18n"
+import {BIBLIOGRAPHY_HEADERS, CATS} from "../../../schema/i18n"
 
 import download from "downloadjs"
+import pretty from "pretty"
 import {DOMSerializer} from "prosemirror-model"
 
 export class HTMLBookExporter extends DOMExporter {
@@ -40,39 +41,63 @@ export class HTMLBookExporter extends DOMExporter {
     }
 
     addBookStyle() {
-        const bookStyle = this.documentStyles.find(bookStyle => bookStyle.slug===this.book.settings.book_style)
+        const bookStyle = this.documentStyles.find(bookStyle => bookStyle.slug === this.book.settings.book_style)
         if (!bookStyle) {
             return false
         }
         // The files will be in the base directory. The filenames of
         // BookStyleFiles will therefore not need to replaced with their URLs.
+        let contents = bookStyle.contents
+        bookStyle.bookstylefile_set.forEach(
+            ([_url, filename]) => contents = contents.replace(
+                new RegExp(filename, 'g'),
+                `media/${filename}`
+            )
+        )
 
-        this.styleSheets.push({contents: bookStyle.contents, filename: `${bookStyle.slug}.css`})
+        this.styleSheets.push({contents, filename: `css/${bookStyle.slug}.css`})
         this.fontFiles = this.fontFiles.concat(bookStyle.bookstylefile_set.map(([url, filename]) => ({
-            filename,
+            filename: `css/media/${filename}`,
             url
         })))
-        return `${bookStyle.slug}.css`
+        return `css/${bookStyle.slug}.css`
     }
 
     exportOne() {
-
         this.chapters = this.book.chapters.sort(
-            (a, b) => a.number > b.number
+            (a, b) => a.number > b.number ? 1 : -1
         ).map(chapter => {
             const doc = this.docList.find(doc => doc.id === chapter.text),
                 schema = this.schema
             schema.cached.imageDB = {db: doc.images}
-            const docContents = removeHidden(doc.contents),
+            const docContent = removeHidden(doc.content),
                 serializer = DOMSerializer.fromSchema(schema),
-                contents = serializer.serializeNode(schema.nodeFromJSON(docContents)),
+                contents = serializer.serializeNode(schema.nodeFromJSON(docContent)),
                 equations = contents.querySelectorAll('.equation'),
                 figureEquations = contents.querySelectorAll('.figure-equation')
             if (equations.length || figureEquations.length) {
                 this.math = true
             }
-
-            contents.querySelectorAll('*[class^="figure-cat-"]').forEach(el => el.innerHTML = FIG_CATS[el.dataset.figureCategory][doc.settings.language])
+            contents.querySelectorAll("figure[data-category='figure'] figcaption span.label").forEach(
+                el => {
+                    el.innerHTML = CATS['figure'][doc.settings.language]
+                }
+            )
+            contents.querySelectorAll("figure[data-category='equation'] figcaption span.label").forEach(
+                el => {
+                    el.innerHTML = CATS['equation'][doc.settings.language]
+                }
+            )
+            contents.querySelectorAll("figure[data-category='photo'] figcaption span.label").forEach(
+                el => {
+                    el.innerHTML = CATS['photo'][doc.settings.language]
+                }
+            )
+            contents.querySelectorAll("figure[data-category='table'] figcaption span.label,table[data-category='table'] caption span.label").forEach(
+                el => {
+                    el.innerHTML = CATS['table'][doc.settings.language]
+                }
+            )
 
             return {
                 doc,
@@ -97,10 +122,10 @@ export class HTMLBookExporter extends DOMExporter {
                     if (bibHTML.length > 0) {
                         chapter.contents.innerHTML += bibHTML
                     }
-                    this.contents = chapter.contents
+                    this.content = chapter.contents
                     this.cleanHTML(citRenderer.fm)
-                    chapter.contents = this.contents
-                    delete this.contents
+                    chapter.contents = this.content
+                    delete this.content
                     return Promise.resolve()
                 }
             )
@@ -116,6 +141,7 @@ export class HTMLBookExporter extends DOMExporter {
     exportTwo() {
         const bookStyle = this.addBookStyle()
         let contentItems = []
+        let currentPart = false
         this.chapters.forEach((chapter, index) => {
             const contents = chapter.contents,
                 doc = chapter.doc,
@@ -126,7 +152,7 @@ export class HTMLBookExporter extends DOMExporter {
             }
             this.prepareBinaryFiles(contents)
 
-            if (this.book.chapters[index].part && this.book.chapters[index].part !== '') {
+            if (this.book.chapters[index].part !== '') {
                 contentItems.push({
                     link: `document-${this.book.chapters[index].number}.html`,
                     title: this.book.chapters[index].part,
@@ -135,17 +161,9 @@ export class HTMLBookExporter extends DOMExporter {
                     level: -1,
                     subItems: []
                 })
+                currentPart = this.book.chapters[index].part
             }
 
-            contentItems.push({
-                link: `document-${this.book.chapters[index].number}.html`,
-                title,
-                docNum: this.book.chapters[index].number,
-                id: 0,
-                level: 0,
-                subItems: []
-            })
-
             // Make links to all H1-3 and create a TOC list of them
             contentItems.push(...setLinks(contents, this.book.chapters[index].number))
 
@@ -153,6 +171,7 @@ export class HTMLBookExporter extends DOMExporter {
 
             const htmlCode = this.chapterTemplate({
                 part: this.book.chapters[index].part,
+                currentPart,
                 title,
                 metadata: doc.metadata,
                 settings: doc.settings,
@@ -163,7 +182,7 @@ export class HTMLBookExporter extends DOMExporter {
 
             this.outputList.push({
                 filename: `document-${this.book.chapters[index].number}.html`,
-                contents: htmlCode
+                contents: pretty(htmlCode, {ocd: true})
             })
         })
 
@@ -171,13 +190,14 @@ export class HTMLBookExporter extends DOMExporter {
 
         this.outputList.push({
             filename: 'index.html',
-            contents: htmlBookIndexTemplate({
+            contents: pretty(htmlBookIndexTemplate({
                 contentItems,
                 book: this.book,
                 creator: this.user.name,
+                styleSheets: [{filename: bookStyle}],
                 // TODO: specify a book language rather than using the current users UI language
                 language: gettext('English')
-            })
+            }), {ocd: true})
         })
 
         return this.exportThree()
@@ -189,7 +209,7 @@ export class HTMLBookExporter extends DOMExporter {
         this.styleSheets = uniqueObjects(this.styleSheets)
         if (this.math) {
             this.includeZips.push({
-                'directory': '',
+                'directory': 'css',
                 'url': `${settings_STATIC_URL}zip/mathlive_style.zip?v=${transpile_VERSION}`
             })
         }
